-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Latha Pillai, Advanced Product Group, Xilinx, Inc.
--  \   \        Filename: DECIMATOR
--  /   /        Date Last Modified:  JUNE23, 2004
-- /___/   /\    Date Created: JUNE23, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-- $Log: $
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 
LIBRARY IEEE;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

ENTITY DECIMATOR IS
   PORT (
      CLK                     : IN std_logic;   
      RST                     : IN std_logic;   
      X_IN                    : IN std_logic_vector(17 DOWNTO 0);   
      OUTPUT                  : OUT std_logic_vector(47 DOWNTO 0));  
END DECIMATOR;

ARCHITECTURE decimation OF DECIMATOR IS


-- DSP48: DSP Function Block
  --        Virtex-4
  --  Xilinx HDL Language Template version 6.1i
   component DSP48
      generic( AREG : integer :=  1;
               BREG : integer :=  1;
               CREG : integer :=  1;
               PREG : integer :=  1;
               MREG : integer :=  1;
               OPMODEREG : integer :=  1;
               SUBTRACTREG : integer :=  1;
               CARRYINSELREG : integer :=  1;
               CARRYINREG : integer :=  1;
               B_INPUT : string :=  "DIRECT";
               LEGACY_MODE : string :=  "MULT18X18S" ;
			SIM_X_INPUT : string := "GENERATE_X_ONLY" );
 port(
        BCOUT                   : out std_logic_vector(17 downto 0);
        P                       : out std_logic_vector(47 downto 0);
        PCOUT                   : out std_logic_vector(47 downto 0);

        A                       : in  std_logic_vector(17 downto 0);
        B                       : in  std_logic_vector(17 downto 0);
        BCIN                    : in  std_logic_vector(17 downto 0);
        C                       : in  std_logic_vector(47 downto 0);
        CARRYIN                 : in  std_ulogic;
        CARRYINSEL              : in  std_logic_vector(1 downto 0);
        CEA                     : in  std_ulogic;
        CEB                     : in  std_ulogic;
        CEC                     : in  std_ulogic;
        CECARRYIN               : in  std_ulogic;
        CECINSUB                : in  std_ulogic;
        CECTRL                  : in  std_ulogic;
        CEM                     : in  std_ulogic;
        CEP                     : in  std_ulogic;
        CLK                     : in  std_ulogic;
        OPMODE                  : in  std_logic_vector(6 downto 0);
        PCIN                    : in  std_logic_vector(47 downto 0);
        RSTA                    : in  std_ulogic;
        RSTB                    : in  std_ulogic;
        RSTC                    : in  std_ulogic;
        RSTCARRYIN              : in  std_ulogic;
        RSTCTRL                 : in  std_ulogic;
        RSTM                    : in  std_ulogic;
        RSTP                    : in  std_ulogic;
        SUBTRACT                : in  std_ulogic
      );
  end component;


   SIGNAL BCOUT1,BCOUT2,BCOUT3                   :  std_logic_vector(17 DOWNTO 0);   
   SIGNAL PCOUT1,PCOUT2,PCOUT3,PCOUT4           :  std_logic_vector(47 DOWNTO 0);   
   SIGNAL TAP1,TAP2,TAP3,TAP4                   :  std_logic_vector(17 DOWNTO 0);   
   SIGNAL CEB1,CEB2,CEB3,CEB4                  : std_logic;  
   SIGNAL AIN1,AIN1_1,AIN1_2,AIN1_3,AIN1_4     :  std_logic_vector(17 DOWNTO 0);   
   SIGNAL AIN2,AIN2_1,AIN2_2,AIN2_3,AIN2_4     :  std_logic_vector(17 DOWNTO 0);   
   SIGNAL AIN3,AIN3_1,AIN3_2,AIN3_3,AIN3_4     :  std_logic_vector(17 DOWNTO 0);   
   SIGNAL AIN4,AIN4_1,AIN4_2,AIN4_3,AIN4_4     :  std_logic_vector(17 DOWNTO 0); 


BEGIN

-- Instantiation block 1
   
    DSP48_1 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 1, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT1,    -- 48-bit cascade output
     A => AIN1,          -- 18-bit A data input
     B => TAP1,          -- 18-bit B data input
     BCIN => "000000000000000000",      -- 18-bit B cascade input
     C => (others => '0'),         -- 48-bit cascade input
     CARRYIN => '0',    -- Carry input signal
     CARRYINSEL => "00", -- 2-bit carry input select
     CEA => '1',        -- A data clock enable input
     CEB => '1',        -- B data clock enable input
     CEC => '0',        -- C data clock enable input
     CECARRYIN => '0',  -- CARRYIN clock enable input
     CECINSUB => '0',   -- CINSUB clock enable input
     CECTRL => '0',     -- Clock Enable input for CTRL registers
     CEM => '1',        -- Clock Enable input for multiplier registers
     CEP => '1',        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => "0000101", -- 7-bit operation mode input
     PCIN => (others => '0'),      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => '0'    -- SUBTRACT input
  );
    -- End of DSP48_1 instantiation

   
   -- Instantiation block 2
   
     DSP48_2 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 1, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT2,    -- 48-bit cascade output
     A => AIN2,          -- 18-bit A data input
     B => TAP2,          -- 18-bit B data input
     BCIN => "000000000000000000",      -- 18-bit B cascade input
     C => (others => '0'),         -- 48-bit cascade input
     CARRYIN => '0',    -- Carry input signal
     CARRYINSEL => "00", -- 2-bit carry input select
     CEA => '1',        -- A data clock enable input
     CEB => '1',        -- B data clock enable input
     CEC => '0',        -- C data clock enable input
     CECARRYIN => '0',  -- CARRYIN clock enable input
     CECINSUB => '0',   -- CINSUB clock enable input
     CECTRL => '0',     -- Clock Enable input for CTRL registers
     CEM => '1',        -- Clock Enable input for multiplier registers
     CEP => '1',        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => "0010101", -- 7-bit operation mode input
     PCIN => PCOUT1,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => '0'    -- SUBTRACT input
  );
    -- End of DSP48_2 instantiation
   
-- Instantiation block 3

    DSP48_3 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 1, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT3,    -- 48-bit cascade output
     A => AIN3,          -- 18-bit A data input
     B => TAP3,          -- 18-bit B data input
     BCIN => "000000000000000000",      -- 18-bit B cascade input
     C => (others => '0'),         -- 48-bit cascade input
     CARRYIN => '0',    -- Carry input signal
     CARRYINSEL => "00", -- 2-bit carry input select
     CEA => '1',        -- A data clock enable input
     CEB => '1',        -- B data clock enable input
     CEC => '0',        -- C data clock enable input
     CECARRYIN => '0',  -- CARRYIN clock enable input
     CECINSUB => '0',   -- CINSUB clock enable input
     CECTRL => '0',     -- Clock Enable input for CTRL registers
     CEM => '1',        -- Clock Enable input for multiplier registers
     CEP => '1',        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => "0010101", -- 7-bit operation mode input
     PCIN => PCOUT2,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => '0'    -- SUBTRACT input
  );
    -- End of DSP48_3 instantiation


   -- Instantiation block 4
   
    DSP48_4 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 1, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT4,    -- 48-bit cascade output
     A => AIN4,          -- 18-bit A data input
     B => TAP4,          -- 18-bit B data input
     BCIN => "000000000000000000",      -- 18-bit B cascade input
     C => (others => '0'),         -- 48-bit cascade input
     CARRYIN => '0',    -- Carry input signal
     CARRYINSEL => "00", -- 2-bit carry input select
     CEA => '1',        -- A data clock enable input
     CEB => '1',        -- B data clock enable input
     CEC => '0',        -- C data clock enable input
     CECARRYIN => '0',  -- CARRYIN clock enable input
     CECINSUB => '0',   -- CINSUB clock enable input
     CECTRL => '0',     -- Clock Enable input for CTRL registers
     CEM => '1',        -- Clock Enable input for multiplier registers
     CEP => '1',        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => "0010101", -- 7-bit operation mode input
     PCIN => PCOUT3,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => '0'    -- SUBTRACT input
  );
    -- End of DSP48_4 instantiation

   
   -- Instantiation block 5
   
    DSP48_5 : DSP48 generic map (
     AREG => 0, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 0, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,    -- 18-bit B cascade output
     P => OUTPUT,          -- 48-bit product output
     PCOUT => open,    -- 48-bit cascade output
     A => "000000000000000000",          -- 18-bit A data input
     B => "000000000000000000",          -- 18-bit B data input
     BCIN => "000000000000000000",      -- 18-bit B cascade input
     C => (others => '0'),         -- 48-bit cascade input
     CARRYIN => '0',    -- Carry input signal
     CARRYINSEL => "00", -- 2-bit carry input select
     CEA => '0',        -- A data clock enable input
     CEB => '0',        -- B data clock enable input
     CEC => '0',        -- C data clock enable input
     CECARRYIN => '0',  -- CARRYIN clock enable input
     CECINSUB => '0',   -- CINSUB clock enable input
     CECTRL => '0',     -- Clock Enable input for CTRL registers
     CEM => '0',        -- Clock Enable input for multiplier registers
     CEP => '1',        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => "0010010", -- 7-bit operation mode input
     PCIN => PCOUT4,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => '0'    -- SUBTRACT input
  );
    -- End of DSP48_5 instantiation
  
   -- 4+1 deep FIFO. Implemented with 5- 18bit shiftregisters 


   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         AIN1 <= "000000000000000000";    
         AIN1_1 <= "000000000000000000";    AIN1_2 <= "000000000000000000";    
         AIN1_3 <= "000000000000000000";    AIN1_4 <= "000000000000000000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         AIN1_1 <= X_IN;    
         AIN1_2 <= AIN1_1;    
         AIN1_3 <= AIN1_2;    
         AIN1_4 <= AIN1_3;    
         AIN1 <= AIN1_4;    
      END IF;
   END PROCESS;

   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         AIN2 <= "000000000000000000";    
         AIN2_1 <= "000000000000000000";    AIN2_2 <= "000000000000000000";    
         AIN2_3 <= "000000000000000000";    AIN2_4 <= "000000000000000000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         AIN2_1 <= AIN1;    
         AIN2_2 <= AIN2_1;    
         AIN2_3 <= AIN2_2;    
         AIN2_4 <= AIN2_3;    
         AIN2 <= AIN2_4;    
      END IF;
   END PROCESS;

   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         AIN3 <= "000000000000000000";    
         AIN3_1 <= "000000000000000000";    AIN3_2 <= "000000000000000000";    
         AIN3_3 <= "000000000000000000";    AIN3_4 <= "000000000000000000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         AIN3_1 <= AIN2;    
         AIN3_2 <= AIN3_1;    
         AIN3_3 <= AIN3_2;    
         AIN3_4 <= AIN3_3;    
         AIN3 <= AIN3_4;    
      END IF;
   END PROCESS;

   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         AIN4 <= "000000000000000000";    
         AIN4_1 <= "000000000000000000";    AIN4_2 <= "000000000000000000";    
         AIN4_3 <= "000000000000000000";    AIN4_4 <= "000000000000000000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         AIN4_1 <= AIN3;    
         AIN4_2 <= AIN4_1;    
         AIN4_3 <= AIN4_2;    
         AIN4_4 <= AIN4_3;    
         AIN4 <= AIN4_4;    
      END IF;
   END PROCESS;

   -- store Coefficients in memory 
   PROCESS (RST, CLK)
      VARIABLE enable  : std_logic_vector(3 DOWNTO 0);
   BEGIN
      IF (RST = '1') THEN
         TAP1 <= "000000000000000000";    
         TAP2 <= "000000000000000000";    
         TAP3 <= "000000000000000000";    
         TAP4 <= "000000000000000000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         enable := CEB1 & CEB2 & CEB3 & CEB4;
         CASE enable IS
            WHEN "1000" =>
                   TAP1 <= "101010101010101010";  
                   TAP2 <= "010101010101010101";  
                   TAP3 <= "000011110000111100";  
                   TAP4 <= "010101010101010101";  
            WHEN "0100" =>
                   TAP1 <= "010101010101010101";  
                   TAP2 <= "101010101010101010";  
                   TAP3 <= "111100001111000011";  
                   TAP4 <= "000011110000111100";  
            WHEN "0010" =>
                   TAP1 <= "111100001111000011";  
                   TAP2 <= "000011110000111100";  
                   TAP3 <= "010101010101010101";  
                   TAP4 <= "111100001111000011";  
            WHEN "0001" =>
                   TAP1 <= "000011110000111100";  
                   TAP2 <= "111100001111000011";  
                   TAP3 <= "101010101010101010";  
                   TAP4 <= "101010101010101010";  
            WHEN OTHERS => NULL;
         END CASE;
      END IF;
      
   END PROCESS;
   
--------------------------------------------------------------------------------

END decimation;
