//------------------------------------------------------------------------------ 
// Copyright (c) 2004 Xilinx, Inc. 
// All Rights Reserved 
//------------------------------------------------------------------------------ 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Author: Latha Pillai, Advanced Product Group, Xilinx, Inc.
//  \   \        Filename: INTERPOLATOR
//  /   /        Date Last Modified:  June 23, 2004 
// /___/   /\    Date Created: June 23, 2004 
// \   \  /  \ 
//  \___\/\___\ 
// 
//
// Revision History: 
// $Log: $
//------------------------------------------------------------------------------ 
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//------------------------------------------------------------------------------ 
/**********************************************************************/


module INTERPOLATOR (CLK, RST, X_IN, OUTPUT);
	
input           CLK, RST;
input   [17:0]  X_IN;
output  [47:0]  OUTPUT;
//output [47:0] ch1,ch2,ch3;

wire[17:0] BCOUT1, BCOUT2, BCOUT3;
wire[47:0] PCOUT1, PCOUT2, PCOUT3;
wire[47:0] OUTPUT;
reg[17:0] TAP1, TAP2, TAP3, TAP4;
reg CEB1, CEB2, CEB3, CEB4;
reg[4:0] CNT16;
 	
//          
// Instantiation block 1
//
  DSP48 DSP48_1 (
      .BCOUT(BCOUT1),  // 18-bit B cascade output
      .P(),          // 48-bit product output
      .PCOUT(PCOUT1),  // 38-bit cascade output
      .A(TAP1),          // 18-bit A data input
      .B(X_IN),          // 18-bit B data input
      .BCIN(18'b0),    // 18-bit B cascade input
      .C(48'b0),          // 48-bit cascade input
      .CARRYIN(1'b0), // Carry input signal
      .CARRYINSEL(2'b0), // 2-bit carry input select
      .CEA(1'b1),      // A data clock enable input
      .CEB(CEB1),      // B data clock enable input
      .CEC(1'b0),      // C data clock enable input
      .CECARRYIN(1'b0), // CARRYIN clock enable input
      .CECINSUB(1'b0), // CINSUB clock enable input
      .CECTRL(1'b0), // Clock Enable input for CTRL registers
      .CEM(1'b1),       // Clock Enable input for multiplier registers
      .CEP(1'b1),       // Clock Enable input for P registers
      .CLK(CLK),       // Clock input
      .OPMODE(7'b0000101), // 7-bit operation mode input
      .PCIN(48'b0),     // 48-bit PCIN input
      .RSTA(RST),     // Reset input for A pipeline registers
      .RSTB(RST),     // Reset input for B pipeline registers
      .RSTC(RST),     // Reset input for C pipeline registers
      .RSTCARRYIN(RST), // Reset input for CARRYIN registers
      .RSTCTRL(RST), // Reset input for CTRL registers
      .RSTM(RST), // Reset input for multiplier registers
      .RSTP(RST), // Reset input for P pipeline registers
      .SUBTRACT(1'b0) // SUBTRACT input
   );
  
   
   //synthesis translate_off
   defparam DSP48_1.AREG = 1; // Number of pipeline registers on the 
                              //   A input, 0, 1 or 2
   defparam DSP48_1.BREG = 2; // Number of pipeline registers on the 
                              //   B input, 0, 1 or 2
   defparam DSP48_1.B_INPUT = "DIRECT"; // B input DIRECT from fabric 
                                        //   or CASCADE from another DSP48
   defparam DSP48_1.CARRYINREG = 0; // Number of pipeline registers 
                                    //   for the CARRYIN input, 0 or 1
   defparam DSP48_1.CARRYINSELREG = 0; // Number of pipeline 
                                        //  registers for the CARRYINSEL, 0 or 1
   defparam DSP48_1.CREG = 0; // Number of pipeline registers on the 
                              //   C input, 0 or 1
   defparam DSP48_1.LEGACY_MODE = "MULT18X18S"; // Backward compatibility, NONE,
                                                //   MULT18X18 or MULT18X18S  
   defparam DSP48_1.MREG = 1; // Number of multiplier pipeline 
                              //   registers, 0 or 1
   defparam DSP48_1.OPMODEREG = 0; // Number of pipeline regsiters on 
                                   //   OPMODE input, 0 or 1
   defparam DSP48_1.PREG = 1; // Number of pipeline registers on the 
                              //   P output, 0 or 1
   defparam DSP48_1.SIM_X_INPUT = "GENERATE_X_ONLY";
                             // Simulation parameter for behavior for X on
                              //   input.
                             //   Possible values: GENERATE_X, NONE or WARNING
   defparam DSP48_1.SUBTRACTREG = 0; // Number of pipeline registers 
                                     //   on the SUBTRACT input, 0 or 1

   //synthesis translate_on
   
   // End of DSP48_1 instantiation 

//          
// Instantiation block 2
//
  DSP48 DSP48_2 (
      .BCOUT(BCOUT2),  // 18-bit B cascade output
      .P(),          // 48-bit product output
      .PCOUT(PCOUT2),  // 38-bit cascade output
      .A(TAP2),          // 18-bit A data input
      .B(18'b0),          // 18-bit B data input
      .BCIN(BCOUT1),    // 18-bit B cascade input
      .C(48'b0),          // 48-bit cascade input
      .CARRYIN(1'b0), // Carry input signal
      .CARRYINSEL(2'b0), // 2-bit carry input select
      .CEA(1'b1),      // A data clock enable input
      .CEB(CEB2),      // B data clock enable input
      .CEC(1'b0),      // C data clock enable input
      .CECARRYIN(1'b0), // CARRYIN clock enable input
      .CECINSUB(1'b0), // CINSUB clock enable input
      .CECTRL(1'b0), // Clock Enable input for CTRL registers
      .CEM(1'b1),       // Clock Enable input for multiplier registers
      .CEP(1'b1),       // Clock Enable input for P registers
      .CLK(CLK),       // Clock input
      .OPMODE(7'b0010101), // 7-bit operation mode input
      .PCIN(PCOUT1),     // 48-bit PCIN input
      .RSTA(RST),     // Reset input for A pipeline registers
      .RSTB(RST),     // Reset input for B pipeline registers
      .RSTC(RST),     // Reset input for C pipeline registers
      .RSTCARRYIN(RST), // Reset input for CARRYIN registers
      .RSTCTRL(RST), // Reset input for CTRL registers
      .RSTM(RST), // Reset input for multiplier registers
      .RSTP(RST), // Reset input for P pipeline registers
      .SUBTRACT(1'b0) // SUBTRACT input
   );
  
   //synthesis translate_off
   defparam DSP48_2.AREG = 1; // Number of pipeline registers on the 
                              //   A input, 0, 1 or 2
   defparam DSP48_2.BREG = 2; // Number of pipeline registers on the 
                              //   B input, 0, 1 or 2
   defparam DSP48_2.B_INPUT = "CASCADE"; // B input DIRECT from fabric 
                                         //  or CASCADE from another DSP48
   defparam DSP48_2.CARRYINREG = 0; // Number of pipeline registers 
                                    //   for the CARRYIN input, 0 or 1
   defparam DSP48_2.CARRYINSELREG = 0; // Number of pipeline 
                                       // registers for the CARRYINSEL, 0 or 1
   defparam DSP48_2.CREG = 0; // Number of pipeline registers on the 
                              //   C input, 0 or 1
   defparam DSP48_2.LEGACY_MODE = "MULT18X18S"; // Backward compatibility, NONE,
                                                //   MULT18X18 or MULT18X18S  
   defparam DSP48_2.MREG = 1; // Number of multiplier pipeline 
                              //   registers, 0 or 1
   defparam DSP48_2.OPMODEREG = 0; // Number of pipeline regsiters on 
                                   //   OPMODE input, 0 or 1
   defparam DSP48_2.PREG = 1; // Number of pipeline registers on the 
                              //   P output, 0 or 1
   defparam DSP48_2.SIM_X_INPUT = "GENERATE_X_ONLY";
                             // Simulation parameter for behavior for X on
                             //    input.
                             //   Possible values: GENERATE_X, NONE or WARNING
   defparam DSP48_2.SUBTRACTREG = 0; // Number of pipeline registers 
                                     //   on the SUBTRACT input, 0 or 1

   //synthesis translate_on
   
   // End of DSP48_2 instantiation 


//          
// Instantiation block 3
//
  DSP48 DSP48_3 (
      .BCOUT(BCOUT3),  // 18-bit B cascade output
      .P(),          // 48-bit product output
      .PCOUT(PCOUT3),  // 38-bit cascade output
      .A(TAP3),          // 18-bit A data input
      .B(18'b0),          // 18-bit B data input
      .BCIN(BCOUT2),    // 18-bit B cascade input
      .C(48'b0),          // 48-bit cascade input
      .CARRYIN(1'b0), // Carry input signal
      .CARRYINSEL(2'b0), // 2-bit carry input select
      .CEA(1'b1),      // A data clock enable input
      .CEB(CEB3),      // B data clock enable input
      .CEC(1'b0),      // C data clock enable input
      .CECARRYIN(1'b0), // CARRYIN clock enable input
      .CECINSUB(1'b0), // CINSUB clock enable input
      .CECTRL(1'b0), // Clock Enable input for CTRL registers
      .CEM(1'b1),       // Clock Enable input for multiplier registers
      .CEP(1'b1),       // Clock Enable input for P registers
      .CLK(CLK),       // Clock input
      .OPMODE(7'b0010101), // 7-bit operation mode input
      .PCIN(PCOUT2),     // 48-bit PCIN input
      .RSTA(RST),     // Reset input for A pipeline registers
      .RSTB(RST),     // Reset input for B pipeline registers
      .RSTC(RST),     // Reset input for C pipeline registers
      .RSTCARRYIN(RST), // Reset input for CARRYIN registers
      .RSTCTRL(RST), // Reset input for CTRL registers
      .RSTM(RST), // Reset input for multiplier registers
      .RSTP(RST), // Reset input for P pipeline registers
      .SUBTRACT(1'b0) // SUBTRACT input
   );
  
   //synthesis translate_off
   defparam DSP48_3.AREG = 1; // Number of pipeline registers on the 
                              //   A input, 0, 1 or 2
   defparam DSP48_3.BREG = 2; // Number of pipeline registers on the 
                              //   B input, 0, 1 or 2
   defparam DSP48_3.B_INPUT = "CASCADE"; // B input DIRECT from fabric 
                                         //  or CASCADE from another DSP48
   defparam DSP48_3.CARRYINREG = 0; // Number of pipeline registers 
                                    //   for the CARRYIN input, 0 or 1
   defparam DSP48_3.CARRYINSELREG = 0; // Number of pipeline 
                                       //   registers for the CARRYINSEL, 0 or 1
   defparam DSP48_3.CREG = 0; // Number of pipeline registers on the 
                              //   C input, 0 or 1
   defparam DSP48_3.LEGACY_MODE = "MULT18X18S"; // Backward compatibility, NONE,
                                                //  MULT18X18 or MULT18X18S  
   defparam DSP48_3.MREG = 1; // Number of multiplier pipeline 
                              //   registers, 0 or 1
   defparam DSP48_3.OPMODEREG = 0; // Number of pipeline regsiters on 
                                   //   OPMODE input, 0 or 1
   defparam DSP48_3.PREG = 1; // Number of pipeline registers on the 
                              //   P output, 0 or 1
   defparam DSP48_3.SIM_X_INPUT = "GENERATE_X_ONLY";
                             // Simulation parameter for behavior for X on
                             //    input.
                             //   Possible values: GENERATE_X, NONE or WARNING
   defparam DSP48_3.SUBTRACTREG = 0; // Number of pipeline registers 
                                     //   on the SUBTRACT input, 0 or 1

   //synthesis translate_on
   
   // End of DSP48_3 instantiation 


//          
// Instantiation block 4
//
  DSP48 DSP48_4 (
      .BCOUT(),  // 18-bit B cascade output
      .P(OUTPUT),          // 48-bit product output
      .PCOUT(),  // 38-bit cascade output
      .A(TAP4),          // 18-bit A data input
      .B(18'b0),          // 18-bit B data input
      .BCIN(BCOUT3),    // 18-bit B cascade input
      .C(48'b0),          // 48-bit cascade input
      .CARRYIN(1'b0), // Carry input signal
      .CARRYINSEL(2'b0), // 2-bit carry input select
      .CEA(1'b1),      // A data clock enable input
      .CEB(CEB4),      // B data clock enable input
      .CEC(1'b0),      // C data clock enable input
      .CECARRYIN(1'b0), // CARRYIN clock enable input
      .CECINSUB(1'b0), // CINSUB clock enable input
      .CECTRL(1'b0), // Clock Enable input for CTRL registers
      .CEM(1'b1),       // Clock Enable input for multiplier registers
      .CEP(1'b1),       // Clock Enable input for P registers
      .CLK(CLK),       // Clock input
      .OPMODE(7'b0010101), // 7-bit operation mode input
      .PCIN(PCOUT3),     // 48-bit PCIN input
      .RSTA(RST),     // Reset input for A pipeline registers
      .RSTB(RST),     // Reset input for B pipeline registers
      .RSTC(RST),     // Reset input for C pipeline registers
      .RSTCARRYIN(RST), // Reset input for CARRYIN registers
      .RSTCTRL(RST), // Reset input for CTRL registers
      .RSTM(RST), // Reset input for multiplier registers
      .RSTP(RST), // Reset input for P pipeline registers
      .SUBTRACT(1'b0) // SUBTRACT input
   );
  
  
   //synthesis translate_off
   defparam DSP48_4.AREG = 1; // Number of pipeline registers on the 
                              //   A input, 0, 1 or 2
   defparam DSP48_4.BREG = 2; // Number of pipeline registers on the 
                              //   B input, 0, 1 or 2
   defparam DSP48_4.B_INPUT = "CASCADE"; // B input DIRECT from fabric 
                                        //   or CASCADE from another DSP48
   defparam DSP48_4.CARRYINREG = 0; // Number of pipeline registers 
                                    //   for the CARRYIN input, 0 or 1
   defparam DSP48_4.CARRYINSELREG = 0; // Number of pipeline 
                                       //   registers for the CARRYINSEL, 0 or 1
   defparam DSP48_4.CREG = 0; // Number of pipeline registers on the 
                              //   C input, 0 or 1
   defparam DSP48_4.LEGACY_MODE = "MULT18X18S"; // Backward compatibility, NONE,
                                                //   MULT18X18 or MULT18X18S  
   defparam DSP48_4.MREG = 1; // Number of multiplier pipeline 
                              //   registers, 0 or 1
   defparam DSP48_4.OPMODEREG = 0; // Number of pipeline regsiters on 
                                   //   OPMODE input, 0 or 1
   defparam DSP48_4.PREG = 1; // Number of pipeline registers on the 
                              //   P output, 0 or 1
   defparam DSP48_4.SIM_X_INPUT = "GENERATE_X_ONLY";
                             // Simulation parameter for behavior for X on
                             //    input.
                             //   Possible values: GENERATE_X, NONE or WARNING
   defparam DSP48_4.SUBTRACTREG = 0; // Number of pipeline registers 
                                      //  on the SUBTRACT input, 0 or 1

   //synthesis translate_on
   
   // End of DSP48_4 instantiation 

/* enable new value in for the first block the first clock after reset
   and for every 4th clock after that for a 4:1 interpolation */

 always @ (posedge CLK or posedge RST)
   begin
   if (RST)
       begin
       CEB1 <= 1'b0; CEB2 <= 1'b0; CEB3 <= 1'b0; CEB4 <= 1'b1;
       end
   else 
       begin
       CEB1 <= CEB4;
       CEB2 <= CEB1;
       CEB3 <= CEB2;
       CEB4 <= CEB3;
       end
   end

/* Load coefficients into register */
/* Load counter */

always @ (posedge CLK or posedge RST)
   begin
   if (RST)
       begin
       CNT16 <= 5'b0;
       end
   else if (CNT16 < 5'b10000)
       begin
       CNT16 <= CNT16 + 1;
       end
   else 
       begin
       CNT16 <= 5'b00000;
       end
   end


/* store Coefficients in memory */

always @ (posedge RST or posedge CLK)
   begin
   if (RST)
       begin
       TAP1 <= 18'd0; TAP2 <= 18'd0; TAP3 <= 18'd0; TAP4 <= 18'd0;
       end
   else
	     case ({CEB1,CEB2,CEB3,CEB4})
         4'b1000 : begin 
                   TAP1 <= 18'd101010101010101010;  
                   TAP2 <= 18'd010101010101010101;  
                   TAP3 <= 18'd000011110000111100;  
                   TAP4 <= 18'd010101010101010101;  end
         4'b0100 : begin 
                   TAP1 <= 18'd010101010101010101;  
                   TAP2 <= 18'd101010101010101010;  
                   TAP3 <= 18'd111100001111000011;  
                   TAP4 <= 18'd000011110000111100;  end
         4'b0010 : begin 
                   TAP1 <= 18'd111100001111000011;  
                   TAP2 <= 18'd000011110000111100;  
                   TAP3 <= 18'd010101010101010101;  
                   TAP4 <= 18'd111100001111000011;  end
         4'b0001 : begin 
                   TAP1 <= 18'd000011110000111100;  
                   TAP2 <= 18'd111100001111000011;  
                   TAP3 <= 18'd101010101010101010;  
                   TAP4 <= 18'd101010101010101010;  end
       endcase
end



//
////////////////////////////////////////////////////////////////////////////////////

endmodule



