-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Latha Pillai, Advanced Product Group, Xilinx, Inc.
--  \   \        Filename: INTERPOLATOR
--  /   /        Date Last Modified:  JUNE23, 2004
-- /___/   /\    Date Created: JUNE23, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-- $Log: $
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 
LIBRARY IEEE;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;

--Library UNISIM;
--use UNISIM.vcomponents.all;

ENTITY INTERPOLATOR IS
   PORT (
      clk                     : IN std_logic; 
      ce                      : IN std_logic;   
      RST                     : IN std_logic; 
      X_IN                    : IN std_logic_vector(17 DOWNTO 0);   
      X_OUT                   : OUT std_logic_vector(47 DOWNTO 0));   
END INTERPOLATOR;


ARCHITECTURE interpolate OF INTERPOLATOR IS

  
-- DSP48: DSP Function Block
  --        Virtex-4
  --  Xilinx HDL Language Template version 6.1i
   component DSP48
      generic( AREG : integer :=  1;
               BREG : integer :=  1;
               CREG : integer :=  1;
               PREG : integer :=  1;
               MREG : integer :=  1;
               OPMODEREG : integer :=  1;
               SUBTRACTREG : integer :=  1;
               CARRYINSELREG : integer :=  1;
               CARRYINREG : integer :=  1;
               B_INPUT : string :=  "DIRECT";
               LEGACY_MODE : string :=  "MULT18X18S") ;
			--SIM_X_INPUT : string := "GENERATE_X_ONLY" );
 port(
        BCOUT                   : out std_logic_vector(17 downto 0);
        P                       : out std_logic_vector(47 downto 0);
        PCOUT                   : out std_logic_vector(47 downto 0);

        A                       : in  std_logic_vector(17 downto 0);
        B                       : in  std_logic_vector(17 downto 0);
        BCIN                    : in  std_logic_vector(17 downto 0);
        C                       : in  std_logic_vector(47 downto 0);
        CARRYIN                 : in  std_logic;
        CARRYINSEL              : in  std_logic_vector(1 downto 0);
        CEA                     : in  std_logic;
        CEB                     : in  std_logic;
        CEC                     : in  std_logic;
        CECARRYIN               : in  std_logic;
        CECINSUB                : in  std_logic;
        CECTRL                  : in  std_logic;
        CEM                     : in  std_logic;
        CEP                     : in  std_logic;
        CLK                     : in  std_logic;
        OPMODE                  : in  std_logic_vector(6 downto 0);
        PCIN                    : in  std_logic_vector(47 downto 0);
        RSTA                    : in  std_logic;
        RSTB                    : in  std_logic;
        RSTC                    : in  std_logic;
        RSTCARRYIN              : in  std_logic;
        RSTCTRL                 : in  std_logic;
        RSTM                    : in  std_logic;
        RSTP                    : in  std_logic;
        SUBTRACT                : in  std_logic
      );
  end component;

SIGNAL BCOUT1, BCOUT2, BCOUT3 :  std_logic_vector(17 DOWNTO 0);
SIGNAL PCOUT1, PCOUT2, PCOUT3 :  std_logic_vector(47 DOWNTO 0);
SIGNAL TAP1, TAP2, TAP3, TAP4 :  std_logic_vector(17 DOWNTO 0);
SIGNAL CEB1, CEB2, CEB3, CEB4 :  std_logic;
SIGNAL ENABLE                 :  std_logic_vector(3 DOWNTO 0);
SIGNAL SET_CARRYINSEL         :  std_logic_vector(1 DOWNTO 0);
SIGNAL SET_18LOW              :  std_logic_vector(17 DOWNTO 0);
SIGNAL SET_48LOW              :  std_logic_vector(47 DOWNTO 0);
SIGNAL SET_HIGH, SET_LOW      :  std_logic;
SIGNAL OPMODE1, OPMODE2, OPMODE3, OPMODE4      : std_logic_vector(6 DOWNTO 0);

BEGIN

SET_HIGH <= '1';
SET_LOW <= '0';
SET_18LOW <= "000000000000000000";
SET_48LOW <= "000000000000000000000000000000000000000000000000";
OPMODE1 <= "0000101";
SET_CARRYINSEL <= "00";

    DSP48_1 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 2, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "DIRECT", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     --SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => BCOUT1,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT1,    -- 48-bit cascade output
     A => TAP1,          -- 18-bit A data input
     B => X_IN,          -- 18-bit B data input
     BCIN => SET_18LOW,      -- 18-bit B cascade input
     C => SET_48LOW,         -- 48-bit cascade input
     CARRYIN => SET_LOW,    -- Carry input signal
     CARRYINSEL => SET_CARRYINSEL, -- 2-bit carry input select
     CEA => SET_HIGH,        -- A data clock enable input
     CEB => CEB1,        -- B data clock enable input
     CEC => SET_LOW,        -- C data clock enable input
     CECARRYIN => SET_LOW,  -- CARRYIN clock enable input
     CECINSUB => SET_LOW,   -- CINSUB clock enable input
     CECTRL => SET_LOW,     -- Clock Enable input for CTRL registers
     CEM => SET_HIGH,        -- Clock Enable input for multiplier registers
     CEP => SET_HIGH,        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE =>OPMODE1, -- 7-bit operation mode input
     PCIN => SET_48LOW,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => SET_LOW    -- SUBTRACT input
  );
    -- End of DSP48_1 instantiation
   
   
OPMODE2 <= "0010101";

DSP48_2 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 2, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "CASCADE", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     --SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => BCOUT2,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT2,    -- 48-bit cascade output
     A => TAP2,         -- 18-bit A data input
     B => SET_18LOW,          -- 18-bit B data input
     BCIN => BCOUT1,     -- 18-bit B cascade input
     C => SET_48LOW,         -- 48-bit cascade input
     CARRYIN => SET_LOW,    -- Carry input signal
     CARRYINSEL => SET_CARRYINSEL, -- 2-bit carry input select
     CEA => SET_HIGH,        -- A data clock enable input
     CEB => CEB2,        -- B data clock enable input
     CEC => SET_LOW,        -- C data clock enable input
     CECARRYIN => SET_LOW,  -- CARRYIN clock enable input
     CECINSUB => SET_LOW,   -- CINSUB clock enable input
     CECTRL => SET_LOW,     -- Clock Enable input for CTRL registers
     CEM => SET_HIGH,        -- Clock Enable input for multiplier registers
     CEP => SET_HIGH,        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE =>OPMODE2, -- 7-bit operation mode input
     PCIN => PCOUT1,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => SET_LOW    -- SUBTRACT input
  );
    -- End of DSP48_2 instantiation

OPMODE3 <= "0010101";

DSP48_3 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 2, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "CASCADE", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
    -- SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => BCOUT3,    -- 18-bit B cascade output
     P => open,          -- 48-bit product output
     PCOUT => PCOUT3,    -- 48-bit cascade output
     A =>TAP3,         -- 18-bit A data input
     B => SET_18LOW,          -- 18-bit B data input
     BCIN => BCOUT2,     -- 18-bit B cascade input
     C => SET_48LOW,         -- 48-bit cascade input
     CARRYIN => SET_LOW,    -- Carry input signal
     CARRYINSEL => SET_CARRYINSEL, -- 2-bit carry input select
     CEA => SET_HIGH,        -- A data clock enable input
     CEB => CEB3,        -- B data clock enable input
     CEC => SET_LOW,        -- C data clock enable input
     CECARRYIN => SET_LOW,  -- CARRYIN clock enable input
     CECINSUB => SET_LOW,   -- CINSUB clock enable input
     CECTRL => SET_LOW,     -- Clock Enable input for CTRL registers
     CEM => SET_HIGH,        -- Clock Enable input for multiplier registers
     CEP => SET_HIGH,        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE =>OPMODE3, -- 7-bit operation mode input
     PCIN => PCOUT2,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => SET_LOW    -- SUBTRACT input
  );
    -- End of DSP48_3 instantiation
   
OPMODE4 <= "0010101";
   
DSP48_4 : DSP48 generic map (
     AREG => 1, -- Number of pipeline registers on the A input, 0, 1 or 2
     BREG => 2, -- Number of pipeline registers on the B input, 0, 1 or 2
     B_INPUT => "CASCADE", -- B input DIRECT from fabric 
                          --or CASCADE from another DSP48
     CARRYINREG => 0, -- Number of pipeline registers 
                      -- for the CARRYIN input, 0 or 1
     CARRYINSELREG => 0, -- Number of pipeline registers 
                         -- for the CARRYINSEL, 0 or 1
     CREG => 0, -- Number of pipeline registers on the C input, 0 or 1
     LEGACY_MODE => "MULT18X18S", -- Backward compatibility, NONE, 
                                  -- MULT18X18 or MULT18X18S
     MREG => 1, -- Number of multiplier pipeline registers, 0 or 1
     OPMODEREG => 0, -- Number of pipeline registers on OPMODE input, 0 or 1
     PREG => 1, -- Number of pipeline registers on the P output, 0 or 1
     --SIM_X_INPUT => "GENERATE_X_ONLY", -- Simulation parameter for 
                                       -- behavior for X on input. Possible 
                                       -- values: GENERATE_X, NONE or WARNING
     SUBTRACTREG => 1) -- Number of pipeline registers on the 
                       -- SUBTRACT input, 0 or 1
  
 port map (
     BCOUT => open,      -- 18-bit B cascade output
     P => X_OUT,         -- 48-bit product output
     PCOUT => OPEN,      -- 48-bit cascade output
     A => TAP4,         -- 18-bit A data input
     B => SET_18LOW,          -- 18-bit B data input
     BCIN => BCOUT3,     -- 18-bit B cascade input
     C => SET_48LOW,         -- 48-bit cascade input
     CARRYIN => SET_LOW,    -- Carry input signal
     CARRYINSEL => SET_CARRYINSEL, -- 2-bit carry input select
     CEA => SET_HIGH,        -- A data clock enable input
     CEB => CEB4,        -- B data clock enable input
     CEC => SET_LOW,        -- C data clock enable input
     CECARRYIN => SET_LOW,  -- CARRYIN clock enable input
     CECINSUB => SET_LOW,   -- CINSUB clock enable input
     CECTRL => SET_LOW,     -- Clock Enable input for CTRL registers
     CEM => SET_HIGH,        -- Clock Enable input for multiplier registers
     CEP => SET_HIGH,        -- Clock Enable input for P registers
     CLK => CLK,         -- Clock input
     OPMODE => OPMODE4, -- 7-bit operation mode input
     PCIN => PCOUT3,      -- 48-bit PCIN input     
     RSTA => RST,        -- Reset input for A pipeline registers
     RSTB => RST,        -- Reset input for B pipeline registers
     RSTC => RST,        -- Reset input for C pipeline registers
     RSTCARRYIN => RST,  -- Reset input for CARRYIN registers
     RSTCTRL => RST,     -- Reset input for CTRL registers
     RSTM => RST,        -- Reset input for multiplier registers
     RSTP => RST,        -- Reset input for P pipeline registers
     SUBTRACT => SET_LOW    -- SUBTRACT input
  );
    -- End of DSP48_4 instantiation

   
   -- enable new value in for the first block the first clock after reset
   -- enable new value in for the first block the first clock after reset
   -- and for every 4th clock after that for a 4:1 interpolation 
   
   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         CEB1 <= SET_LOW;    
         CEB2 <= SET_LOW;    
         CEB3 <= SET_LOW;    
         CEB4 <= SET_HIGH;    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         CEB1 <= CEB4;    
         CEB2 <= CEB1;    
         CEB3 <= CEB2;    
         CEB4 <= CEB3;    
      END IF;
   END PROCESS;

 ENABLE <= CEB1 & CEB2 & CEB3 & CEB4;

   -- store Coefficients in memory 
   PROCESS (RST, CLK)
   BEGIN
      IF (RST = '1') THEN
         TAP1 <= (others => '0');    
         TAP2 <= (others => '0');    
         TAP3 <= (others => '0');    
         TAP4 <= (others => '0');    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         CASE ENABLE IS
            WHEN "1000" =>
                   TAP1 <= "101010101010101010";  
                   TAP2 <= "010101010101010101";  
                   TAP3 <= "000011110000111100";  
                   TAP4 <= "010101010101010101";  
            WHEN "0100" =>
                   TAP1 <= "010101010101010101";  
                   TAP2 <= "101010101010101010";  
                   TAP3 <= "111100001111000011";  
                   TAP4 <= "000011110000111100";  
            WHEN "0010" =>
                   TAP1 <= "111100001111000011";  
                   TAP2 <= "000011110000111100";  
                   TAP3 <= "010101010101010101";  
                   TAP4 <= "111100001111000011";  
            WHEN "0001" =>
                   TAP1 <= "000011110000111100";  
                   TAP2 <= "111100001111000011";  
                   TAP3 <= "101010101010101010";  
                   TAP4 <= "101010101010101010";  
            WHEN OTHERS => NULL; 
         END CASE;
      END IF;
   END PROCESS;
   
----------------------------------------------------------------
END interpolate;
